# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

from functools import reduce


class HueCommand:
    @staticmethod
    def get_by_unique_id(unique_id: str, items: list) -> list:
        return list(
            filter(
                lambda info: "uniqueid" in info and info["uniqueid"] == unique_id, items
            )
        )[0]

    @staticmethod
    def _process(value):
        print(value)

    @staticmethod
    def _mapper(path: str, value_type):
        return lambda value: value_type(
            reduce(lambda p, field: p[field], path.split("."), value)
        )

    @staticmethod
    def name():
        pass

    def exec(self, arguments):
        pass



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

from huemon.api.api_interface import ApiInterface
from huemon.commands.hue_command_interface import HueCommand
from huemon.infrastructure.logger_factory import create_logger
from huemon.util import assert_exists, assert_num_args

LOG = create_logger()


class SensorCommand(HueCommand):
    def __init__(
        self, config: dict, api: ApiInterface
    ):  # pylint: disable=unused-argument
        self.api = api

    def __get_sensor(self, device_id):
        return HueCommand.get_by_unique_id(device_id, self.api.get_sensors())

    def __map_sensor(self, unique_id, mapper):
        return mapper(self.__get_sensor(unique_id))

    __SENSOR_ACTION_MAP = {
        "battery:level": HueCommand._mapper("config.battery", float),
        "light:level": HueCommand._mapper("state.lightlevel", float),
        "presence": HueCommand._mapper("state.presence", int),
        "reachable": HueCommand._mapper("config.reachable", int),
        "temperature": lambda device: float(device["state"]["temperature"] / 100),
    }

    @staticmethod
    def name():
        return "sensor"

    def exec(self, arguments):
        LOG.debug(
            "Running `%s` command (arguments=%s)", SensorCommand.name(), arguments
        )
        assert_num_args(2, arguments, SensorCommand.name())

        device_id, action = arguments

        assert_exists(list(SensorCommand.__SENSOR_ACTION_MAP), action)

        HueCommand._process(
            self.__map_sensor(device_id, SensorCommand.__SENSOR_ACTION_MAP[action])
        )
        LOG.debug(
            "Finished `%s` command (arguments=%s)", SensorCommand.name(), arguments
        )



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

import json
from functools import reduce


class Discovery:
    @staticmethod
    def _item_to_discovery(item: dict):
        return {
            "{#NAME}": item["name"],
            "{#UNIQUE_ID}": item["uniqueid"],
        }

    @staticmethod
    def _has_state_field(field: str):
        return (
            lambda item: "state" in item
            and field in item["state"]
            and "recycle" not in item
        )

    @staticmethod
    def _print_array_as_discovery(items):
        print(
            json.dumps(
                {
                    "data": reduce(
                        lambda p, item: [*p, Discovery._item_to_discovery(item)],
                        items,
                        [],
                    )
                }
            )
        )

    @staticmethod
    def name():
        pass

    def exec(self, arguments=None):
        pass



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

from huemon.api.api_interface import ApiInterface
from huemon.discoveries.discovery_interface import Discovery


class BatteriesDiscovery(Discovery):
    def __init__(self, api: ApiInterface):
        self.api = api

    @staticmethod
    def name():
        return "batteries"

    def exec(self, arguments=None):
        Discovery._print_array_as_discovery(self.api.get_batteries())



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

from huemon.api.api_interface import ApiInterface
from huemon.commands.hue_command_interface import HueCommand
from huemon.infrastructure.logger_factory import create_logger
from huemon.util import assert_exists, assert_num_args

LOG = create_logger()


class SystemCommand(HueCommand):
    def __init__(
        self, config: dict, api: ApiInterface
    ):  # pylint: disable=unused-argument
        self.api = api

    def __map_config(self, mapper):
        return mapper(self.api.get_system_config())

    __SYSTEM_ACTION_MAP = {
        "is_upgrade_available": lambda config: int(
            config["swupdate2"]["state"] != "noupdates"
        ),
        "version": HueCommand._mapper("swversion", str),
    }

    @staticmethod
    def name():
        return "system"

    def exec(self, arguments):
        LOG.debug(
            "Running `%s` command (arguments=%s)", SystemCommand.name(), arguments
        )
        assert_num_args(1, arguments, SystemCommand.name())

        action, *_ = arguments

        assert_exists(list(SystemCommand.__SYSTEM_ACTION_MAP), action)

        HueCommand._process(self.__map_config(self.__SYSTEM_ACTION_MAP[action]))
        LOG.debug(
            "Finished `%s` command (arguments=%s)", SystemCommand.name(), arguments
        )



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

import os
from pathlib import Path

from genericpath import isdir, isfile

from huemon.commands.hue_command_interface import HueCommand
from huemon.infrastructure.logger_factory import create_logger
from huemon.util import (
    assert_num_args,
    exit_fail,
    get_commands_path,
    get_discoveries_path,
)

LOG = create_logger()


def symlink_plugins(plugins_available_path, plugins_enabled_path):
    for plugins_path in [plugins_available_path, plugins_enabled_path]:
        if not isdir(plugins_path):
            exit_fail("Provided path `%s` does not exist", plugins_path)

    available_plugins = list(Path(plugins_available_path).glob("*.py"))

    LOG.debug(
        "Symlinking %s plugins (from=%s,to=%s)",
        len(available_plugins),
        plugins_available_path,
        plugins_enabled_path,
    )
    for plugin_path in available_plugins:
        target_path = os.path.join(plugins_enabled_path, plugin_path.name)

        if isfile(target_path):
            LOG.debug("Skipping `%s`, symlink already exists", target_path)
            continue

        os.symlink(plugin_path.absolute(), target_path)
    LOG.debug(
        "Finished symlinking %s plugins (from=%s,to=%s)",
        len(available_plugins),
        plugins_available_path,
        plugins_enabled_path,
    )


class InstallAvailableCommand(HueCommand):
    def __init__(self, config: dict):
        self.config = config

    @staticmethod
    def name():
        return "install_available"

    def exec(self, arguments):
        LOG.debug(
            "Running `%s` command (arguments=%s)",
            InstallAvailableCommand.name(),
            arguments,
        )
        assert_num_args(1, arguments, InstallAvailableCommand.name())

        target, *_ = arguments

        if target not in ["commands", "discoveries"]:
            exit_fail(
                "Received unknown action `%s` for `%s` command",
                target,
                InstallAvailableCommand.name(),
            )

        if target == "commands":
            commands_enabled_path = get_commands_path(self.config, "enabled")
            commands_available_path = get_commands_path(self.config, "available")

            symlink_plugins(commands_available_path, commands_enabled_path)

        if target == "discoveries":
            discoveries_enabled_path = get_discoveries_path(self.config, "enabled")
            discoveries_available_path = get_discoveries_path(self.config, "available")

            symlink_plugins(discoveries_available_path, discoveries_enabled_path)

        LOG.debug(
            "Finished `%s` command (arguments=%s)",
            InstallAvailableCommand.name(),
            arguments,
        )



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

import importlib.util
import inspect
from pathlib import Path


def __get_plugin_type(module_name: str, path: str, sub_class):
    maybe_spec = importlib.util.spec_from_file_location(module_name, path)
    if not maybe_spec:
        return None

    module = importlib.util.module_from_spec(maybe_spec)

    if not maybe_spec.loader:
        return None

    maybe_spec.loader.exec_module(module)

    plugin_types = list(
        filter(
            lambda m: inspect.isclass(m[1])
            and issubclass(m[1], sub_class)
            and m[1] is not sub_class,
            inspect.getmembers(module),
        )
    )

    if len(plugin_types) == 0:
        return None

    _, hue_command_class = plugin_types[0]

    return hue_command_class


def load_plugins(module_name: str, path: str, plugin_type):
    return list(
        map(
            lambda p: __get_plugin_type(
                f"{module_name}.{p.stem}", str(p.absolute()), plugin_type
            ),
            Path(path).glob("*.py"),
        )
    )



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

from huemon.api.api_interface import ApiInterface
from huemon.commands.hue_command_interface import HueCommand
from huemon.infrastructure.logger_factory import create_logger
from huemon.util import assert_exists, assert_num_args

LOG = create_logger()


class LightCommand(HueCommand):
    def __init__(
        self, config: dict, api: ApiInterface
    ):  # pylint: disable=unused-argument
        self.api = api

    __LIGHT_ACTION_MAP = {
        "is_upgrade_available": lambda light: int(
            light["swupdate"]["state"] != "noupdates"
        ),
        "reachable": HueCommand._mapper("state.reachable", int),
        "status": HueCommand._mapper("state.on", int),
        "version": HueCommand._mapper("swversion", str),
    }

    def __get_light(self, unique_id):
        return HueCommand.get_by_unique_id(unique_id, self.api.get_lights())

    def __map_light(self, unique_id, mapper):
        return mapper(self.__get_light(unique_id))

    @staticmethod
    def name():
        return "light"

    def exec(self, arguments):
        LOG.debug("Running `%s` command (arguments=%s)", LightCommand.name(), arguments)
        assert_num_args(2, arguments, LightCommand.name())

        light_id, action = arguments

        assert_exists(list(LightCommand.__LIGHT_ACTION_MAP), action)

        HueCommand._process(
            self.__map_light(light_id, LightCommand.__LIGHT_ACTION_MAP[action])
        )

        LOG.debug(
            "Finished `%s` command (arguments=%s)", LightCommand.name(), arguments
        )



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

import json
import os
import tempfile
import time
from fcntl import LOCK_EX, LOCK_NB, flock
from os.path import exists

from huemon.api.api_interface import ApiInterface
from huemon.infrastructure.logger_factory import create_logger

LOG = create_logger()

DEFAULT_MAX_CACHE_AGE_SECONDS = 10
DEFAULT_CACHE_PATH = tempfile.gettempdir()


class CachedApi(ApiInterface):
    def __init__(
        self,
        api: ApiInterface,
        max_cache_age_seconds=DEFAULT_MAX_CACHE_AGE_SECONDS,
        cache_path=DEFAULT_CACHE_PATH,
    ):
        self.api = api
        self.cache_path = cache_path
        self.max_cache_age_seconds = max_cache_age_seconds

    def __tf(self, filename):
        return "/".join([self.cache_path, filename])

    def __cache(self, resource_type: str, fn_call):
        temp_filename = f"zabbix-hue.{resource_type}"
        cache_file_path = self.__tf(f"{temp_filename}.json")
        lock_file = self.__tf(f"{temp_filename}.lock")

        does_cache_file_exist = exists(cache_file_path)
        cache_age_seconds = (
            time.time() - os.path.getmtime(cache_file_path)
            if does_cache_file_exist
            else 0
        )
        is_cache_file_expired = (
            not does_cache_file_exist or cache_age_seconds >= self.max_cache_age_seconds
        )
        is_cache_hit = not is_cache_file_expired

        if is_cache_hit:
            with open(cache_file_path, "r") as f_json:
                LOG.debug(
                    "Cache hit (type=%s,age_seconds=%s,max_cache_age_seconds=%s)",
                    resource_type,
                    round(cache_age_seconds, 1),
                    self.max_cache_age_seconds,
                )
                return json.loads(f_json.read())

        with open(lock_file, "w") as f_lock:
            try:
                flock(f_lock.fileno(), LOCK_EX | LOCK_NB)
                LOG.debug("Acquired lock successfully (file=%s)", lock_file)

                tmp_fd, tmp_file_path = tempfile.mkstemp()
                with open(tmp_file_path, "w") as f_tmp:
                    f_tmp.write(json.dumps(fn_call()))

                os.close(tmp_fd)

                os.rename(tmp_file_path, cache_file_path)

                with open(cache_file_path) as f_json:
                    return json.loads(f_json.read())
            except:  # pylint: disable=bare-except
                LOG.debug("Failed to acquire lock, cache hit (file=%s)", lock_file)

        if not does_cache_file_exist:
            return []

        with open(cache_file_path) as f_json:
            return json.loads(f_json.read())

    def get_system_config(self):
        return self.__cache("system", self.api.get_system_config)

    def get_lights(self):
        return self.__cache("lights", self.api.get_lights)

    def get_sensors(self):
        return self.__cache("sensors", self.api.get_sensors)

    def get_batteries(self):
        return self.__cache("batteries", self.api.get_batteries)



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

from os import environ, path
from pathlib import Path

import yaml
from genericpath import isfile

from huemon.util import exit_fail

CONFIG_PATH_LOCAL = path.join(str(Path(__file__).parent.parent), "config.yml")
CONFIG_PATH_ENV_VARIABLE = environ.get("HUEMON_CONFIG_PATH")

CONFIG_PATHS_ORDERED_PREFERENCE = list(
    filter(
        None,
        [
            CONFIG_PATH_ENV_VARIABLE,
            CONFIG_PATH_LOCAL,
        ],
    )
)


def __first_existing_config_file():
    for config_path in CONFIG_PATHS_ORDERED_PREFERENCE:
        if isfile(config_path):
            return config_path

    return None


def create_config():
    maybe_config_path = __first_existing_config_file()
    if not maybe_config_path:
        exit_fail(
            "No configuration file found in %s",
            ",".join(CONFIG_PATHS_ORDERED_PREFERENCE),
        )

    with open(maybe_config_path, "r") as file:
        return yaml.safe_load(file.read())



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

from huemon.api.api_interface import ApiInterface
from huemon.discoveries.discovery_interface import Discovery
from huemon.infrastructure.logger_factory import create_logger
from huemon.util import exit_fail

LOG = create_logger()

TYPE_LIGHT = "light"
TYPE_PRESENCE = "presence"
TYPE_TEMPERATURE = "temperature"

SENSOR_TYPES = [TYPE_LIGHT, TYPE_PRESENCE, TYPE_TEMPERATURE]


class SensorsDiscovery(Discovery):
    def __init__(self, api: ApiInterface):
        self.api = api

    @staticmethod
    def name():
        return "sensors"

    def exec(self, arguments=None):
        if not arguments or len(arguments) == 0:
            exit_fail(
                "Did not receive enough arguments for `discover sensor:*`, expected 1 received 0"
            )

        sensor_type, *_ = arguments

        LOG.debug("Running `discover sensor:*` command (sensor_type=%s)", sensor_type)
        if sensor_type not in SENSOR_TYPES:
            exit_fail(
                "Received unknown sensor type '%s' for `discover sensor:*` command (expected=%s)",
                sensor_type,
                SENSOR_TYPES,
            )

        Discovery._print_array_as_discovery(
            filter(
                Discovery._has_state_field(
                    "lightlevel" if sensor_type == "light" else sensor_type
                ),
                self.api.get_sensors(),
            )
        )
        LOG.debug("Finished `discover sensor:*` command (sensor_type=%s)", sensor_type)



# By Jonathan Bowman
# https://dev.to/bowmanjd/hardening-and-simplifying-python-s-urlopen-4gee dd 20220302@1410

import typing
import urllib.request

from huemon.infrastructure.logger_factory import create_logger

LOG = create_logger()


class WhitelistedSchemaOpener(urllib.request.OpenerDirector):
    def __init__(self, handlers: typing.Iterable = None):
        super().__init__()
        handlers = handlers or (
            urllib.request.UnknownHandler,
            urllib.request.HTTPHandler,
            urllib.request.HTTPDefaultErrorHandler,
            urllib.request.HTTPRedirectHandler,
            urllib.request.HTTPSHandler,
            urllib.request.HTTPErrorProcessor,
        )

        for handler_class in handlers:
            self.add_handler(handler_class())


def security_urllib_allow_http_and_https_schemas_only():
    LOG.debug("Limiting urllib.request to http and https schemas")
    urllib.request.install_opener(WhitelistedSchemaOpener())
    LOG.debug("Limited urllib.request to http and https schemas")



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

import logging
import logging.config

DEFAULT_LOGGER_NAME = "hue"


def bootstrap_logger(config: dict, name=DEFAULT_LOGGER_NAME):
    logging.config.dictConfig(config)

    return create_logger(name)


def create_logger(name=DEFAULT_LOGGER_NAME):
    return logging.getLogger(name)



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

from functools import reduce

from huemon.api.api_factory import create_api
from huemon.api.api_interface import ApiInterface
from huemon.commands.hue_command_interface import HueCommand
from huemon.commands_internal.install_available_command import InstallAvailableCommand
from huemon.infrastructure.logger_factory import create_logger
from huemon.plugin_loader import load_plugins
from huemon.util import exit_fail

LOG = create_logger()


def create_name_to_command_mapping(config: dict, api: ApiInterface, plugins: list):
    return reduce(lambda p, c: {**p, c.name(): c(config, api)}, plugins, {})


def __load_command_plugins(config: dict, command_plugins_path: str):
    LOG.debug("Loading command plugins (path=%s)", command_plugins_path)
    command_handler_plugins = create_name_to_command_mapping(
        config,
        create_api(config),
        load_plugins("command", command_plugins_path, HueCommand),
    )
    LOG.debug("Finished loading command plugins (path=%s)", command_plugins_path)

    return command_handler_plugins


def __load_plugins_and_hardwired_handlers(config: dict, command_plugins_path: str):
    return {
        **__load_command_plugins(config, command_plugins_path),
        InstallAvailableCommand.name(): InstallAvailableCommand(config),
    }


def create_default_command_handler(config: dict, command_plugins_path: str):
    return CommandHandler(
        __load_plugins_and_hardwired_handlers(config, command_plugins_path)
    )


class CommandHandler:  # pylint: disable=too-few-public-methods
    def __init__(self, handlers):
        self.handlers = handlers

    def available_commands(self):
        return list(self.handlers)

    def exec(self, command: str, arguments):
        LOG.debug("Running command `%s` (arguments=%s)", command, arguments)
        if not command in self.handlers:
            exit_fail(
                "Received unknown command `%s`, expected one of %s",
                command,
                self.available_commands(),
            )

        self.handlers[command].exec(arguments)

        LOG.debug("Finished command `%s` (arguments=%s)", command, arguments)



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.


class ApiInterface:
    def get_system_config(self):
        pass

    def get_lights(self):
        pass

    def get_sensors(self):
        pass

    def get_batteries(self):
        pass



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

import json
from urllib.request import urlopen

from huemon.api.api_interface import ApiInterface
from huemon.infrastructure.logger_factory import create_logger

LOG = create_logger()


class Api(ApiInterface):
    def __init__(self, hub_url: str):
        self.hub_url = hub_url

    def __hub_url(self, relative_url):
        return "/".join([self.hub_url, relative_url])

    def __get_resource(self, resource):
        # B310:blacklist audit url open for permitted schemes handled by urllib_safe_opener
        with urlopen(self.__hub_url(resource)) as response:  # nosec
            LOG.debug("Retrieving `%s` from api", resource)
            json_response = json.loads(response.read())
            LOG.debug("Retrieved `%s` from api", resource)
            return json_response

    def get_system_config(self):
        return self.__get_resource("config")

    def get_lights(self):
        return list(self.__get_resource("lights").values())

    def get_sensors(self):
        return list(self.__get_resource("sensors").values())

    def get_batteries(self):
        return list(
            filter(
                lambda s: "config" in s and "battery" in s["config"],
                list(self.get_sensors()),
            )
        )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['huemon',
 'huemon.api',
 'huemon.commands',
 'huemon.commands_available',
 'huemon.commands_internal',
 'huemon.discoveries',
 'huemon.discoveries_available',
 'huemon.infrastructure']

package_data = \
{'': ['*'],
 'huemon': ['commands_enabled/.gitkeep', 'discoveries_enabled/.gitkeep']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'mypy>=0.931,<0.932',
 'nox>=2022.1.7,<2023.0.0',
 'types-PyYAML>=6.0.4,<7.0.0']

setup_kwargs = {
    'name': 'huemon',
    'version': '0.3.0',
    'description': 'Monitor your Philips Hue network',
    'long_description': '# Huemon\n\n[![License: MPL 2.0](https://img.shields.io/badge/License-MPL%202.0-brightgreen.svg)](https://opensource.org/licenses/MPL-2.0)\n[![Build](https://github.com/edeckers/huemon/actions/workflows/test.yml/badge.svg)](https://github.com/edeckers/huemon/actions/workflows/test.yml)\n[![security: bandit](https://img.shields.io/badge/security-bandit-yellow.svg)](https://github.com/PyCQA/bandit)\n\nZabbix monitoring for Philips Hue networks.\n\n![Dashboard: sensors](https://raw.githubusercontent.com/edeckers/huemon/develop/docs/assets/dashboard-sensors.png?raw=true "Dashboard: sensors")\n\n## Requirements\n\n- Zabbix server 5.0+\n- Zabbix agent 5.0+\n- Python 3.0+ on Zabbix agent machine\n\n## Installation\n\n```bash\npip3 install huemon\n```\n\n## Configuration\n\n1. Copy `config.example.yml` from `src/huemon` to `/path/to/config.yml`\n2. Make necessary changes\n3. Provide the path through environment variable `HUEMON_CONFIG_PATH`\n\n### Enabling commands and discoveries\n\n#### Automatically\n\n```bash\nHUEMON_CONFIG_PATH=/path/to/config.yml python3 -m huemon install_available commands\nHUEMON_CONFIG_PATH=/path/to/config.yml python3 -m huemon install_available discoveries\n```\n\n#### Manually\n\n```bash\nln -s /path/to/commands_available/command_name.py /path/to/commands_enabled/command_name.py\nln -s /path/to/discoveries_available/command_name.py /path/to/discoveries_enabled/command_name.py\n```\n\n## Usage\n\n### Shell\n\n```bash\nHUEMON_CONFIG_PATH=/usr/bin/python3 -m huemon discover lights\n```\n\n### Docker\n\n```bash\ndocker run -v /path/to/huemon/config:/etc/huemon huemon:0.1.0 discover lights\n```\n\n### Zabbix agent configuration\n\n```\n# file:/path/to/zabbix/agent/conf.d/hue.conf\n\nUserParameter=hue.discovery[*],HUEMON_CONFIG_PATH=/usr/bin/python3 -m huemon discover $1\nUserParameter=hue.value[*],HUEMON_CONFIG_PATH=/usr/bin/python3 -m huemon $1 $2 $3\n```\n\n## Screenshots\n\n### Dashboards\n![Dashboard: sensors](https://raw.githubusercontent.com/edeckers/huemon/develop/docs/assets/dashboard-sensors.png?raw=true "Dashboard: sensors")\n\n### Discoveries\n\n![Discoveries: batteries](https://raw.githubusercontent.com/edeckers/huemon/develop/docs/assets/discoveries-batteries.png?raw=true "Discoveries: batteries")\n\n![Discoveries: lights](https://raw.githubusercontent.com/edeckers/huemon/develop/docs/assets/discoveries-lights.png?raw=true "Discoveries: lights")\n\n![Discoveries: sensors](https://raw.githubusercontent.com/edeckers/huemon/develop/docs/assets/discoveries-sensors.png?raw=true "Discoveries: sensors")\n\n### Template\n\n![Template](https://raw.githubusercontent.com/edeckers/huemon/develop/docs/assets/template-discoveries.png?raw=true "Template")\n\n\n## License\n\nMPL-2.0\n',
    'author': 'Ely Deckers',
    'author_email': None,
    'maintainer': 'Ely Deckers',
    'maintainer_email': None,
    'url': 'https://github.com/edeckers/huemon.git',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from huemon.api.api import Api
from huemon.api.cached_api import CachedApi
from huemon.const import DEFAULT_MAX_CACHE_AGE_SECONDS


def create_hue_hub_url(config: dict):
    return f"http://{config['ip']}/api/{config['key']}"


def create_api(config: dict):
    enable_cache = (
        "cache" in config
        and "enable" in config["cache"]
        and bool(config["cache"]["enable"])
    )

    is_cache_age_configured = "cache" in config and "max_age_seconds" in config["cache"]

    max_cache_age_seconds = (
        int(config["cache"]["max_age_seconds"])
        if is_cache_age_configured
        else DEFAULT_MAX_CACHE_AGE_SECONDS
    )

    api = Api(create_hue_hub_url(config))

    return CachedApi(api, max_cache_age_seconds) if enable_cache else api



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

from functools import reduce

from huemon.api.api_interface import ApiInterface
from huemon.commands.hue_command_interface import HueCommand
from huemon.discoveries.discovery_interface import Discovery
from huemon.infrastructure.logger_factory import create_logger
from huemon.plugin_loader import load_plugins
from huemon.util import assert_exists, assert_num_args, get_discoveries_path

LOG = create_logger()


def create_discovery_handlers(api: ApiInterface, plugins: dict):
    return reduce(lambda p, c: {**p, c.name(): c(api)}, plugins, {})


class DiscoveryHandler:  # pylint: disable=too-few-public-methods
    def __init__(self, handlers):
        self.handlers = handlers

    def exec(self, discovery_type):
        LOG.debug(
            "Running `%s` command (discovery_type=%s)",
            DiscoverCommand.name(),
            discovery_type,
        )
        target, maybe_sub_target, *_ = discovery_type.split(":") + [None]

        assert_exists(list(self.handlers), target)

        self.handlers[target].exec([maybe_sub_target] if maybe_sub_target else [])

        LOG.debug(
            "Finished `%s` command (discovery_type=%s)",
            DiscoverCommand.name(),
            discovery_type,
        )


class Discover:  # pylint: disable=too-few-public-methods
    def __init__(self, config: dict, api: ApiInterface):
        self.api = api

        self.discovery_plugins_path = get_discoveries_path(config, "enabled")

    def discover(self, discovery_type):
        LOG.debug("Loading command plugins (path=%s)", self.discovery_plugins_path)
        discovery_handler_plugins = create_discovery_handlers(
            self.api, load_plugins("discovery", self.discovery_plugins_path, Discovery)
        )
        LOG.debug(
            "Finished loading command plugins (path=%s)", self.discovery_plugins_path
        )

        DiscoveryHandler(discovery_handler_plugins).exec(discovery_type)


class DiscoverCommand(HueCommand):
    def __init__(self, config: dict, api: ApiInterface):
        self.discovery = Discover(config, api)

    @staticmethod
    def name():
        return "discover"

    def exec(self, arguments):
        LOG.debug(
            "Running `%s` command (arguments=%s)", DiscoverCommand.name(), arguments
        )
        assert_num_args(1, arguments, DiscoverCommand.name())

        discovery_type, *_ = arguments

        self.discovery.discover(discovery_type)
        LOG.debug("Finished `discover` command (arguments=%s)", arguments)



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

DEFAULT_MAX_CACHE_AGE_SECONDS = 10

EXIT_OK = 0
EXIT_FAIL = 1



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

import os
import sys
from pathlib import Path

from huemon.const import EXIT_FAIL
from huemon.infrastructure.logger_factory import create_logger


def create_local_path(relative_path: str):
    return str(os.path.join(Path(__file__).parent.absolute(), relative_path))


LOG = create_logger()

DEFAULT_COMMANDS_AVAILABLE_PATH = create_local_path("commands_available")
DEFAULT_COMMANDS_ENABLED_PATH = create_local_path("commands_enabled")
DEFAULT_DISCOVERIES_AVAILABLE_PATH = create_local_path("discoveries_available")
DEFAULT_DISCOVERIES_ENABLED_PATH = create_local_path("discoveries_enabled")

PLUGIN_TYPE_MAPPING = {
    "commands": {
        "available": DEFAULT_COMMANDS_AVAILABLE_PATH,
        "enabled": DEFAULT_COMMANDS_ENABLED_PATH,
    },
    "discoveries": {
        "available": DEFAULT_DISCOVERIES_AVAILABLE_PATH,
        "enabled": DEFAULT_DISCOVERIES_ENABLED_PATH,
    },
}


def __create_plugins_path(
    plugin_type: str, config: dict, path_type: str, fallback_path: str = None
):
    return (
        config[plugin_type][path_type]
        if plugin_type in config and path_type in config[plugin_type]
        else (fallback_path or PLUGIN_TYPE_MAPPING[plugin_type][path_type])
    )


def get_commands_path(config: dict, path_type: str, fallback_path: str = None):
    return __create_plugins_path("commands", config, path_type, fallback_path)


def get_discoveries_path(config: dict, path_type: str, fallback_path: str = None):
    return __create_plugins_path("discoveries", config, path_type, fallback_path)


def exit_fail(message, *arguments):
    LOG.error(message, *arguments)
    print(message % tuple(arguments))

    sys.exit(EXIT_FAIL)


def assert_num_args(expected_number_of_arguments: int, arguments: list, context: str):
    if len(arguments) != expected_number_of_arguments:
        argument_text = "argument" if expected_number_of_arguments == 1 else "arguments"

        exit_fail(
            "Expected exactly %s %s for `%s`, received %s",
            expected_number_of_arguments,
            argument_text,
            context,
            len(arguments),
        )


def assert_exists(expected_values: list, value: str):
    if value not in expected_values:
        exit_fail("Received unknown value `%s` (expected=%s)", value, expected_values)



# Copyright (c) Ely Deckers.
#
# This source code is licensed under the MPL-2.0 license found in the
# LICENSE file in the root directory of this source tree.

import sys

from huemon.commands.command_handler import create_default_command_handler
from huemon.const import EXIT_OK
from huemon.infrastructure.config_factory import create_config
from huemon.infrastructure.logger_factory import bootstrap_logger
from huemon.infrastructure.urllib_safe_opener import (
    security_urllib_allow_http_and_https_schemas_only,
)
from huemon.util import exit_fail, get_commands_path

CONFIG = create_config()
LOG = bootstrap_logger(CONFIG)


class Main:  # pylint: disable=too-few-public-methods
    @staticmethod
    def __bootstrap():
        LOG.debug("Bootstrapping application")
        security_urllib_allow_http_and_https_schemas_only()
        LOG.debug("Finished bootstrapping application")

    @staticmethod
    def main(argv):
        LOG.debug("Running script (parameters=%s)", argv[1:])
        Main.__bootstrap()

        command_handler = create_default_command_handler(
            CONFIG, get_commands_path(CONFIG, "enabled")
        )

        if len(argv) <= 1:
            exit_fail(
                "Did not receive enough arguments, expected one of %s (arguments=%s)",
                command_handler.available_commands(),
                argv[1:],
            )

        command, *arguments = argv[1:]

        command_handler.exec(command, arguments)

        LOG.debug("Finished script (parameters=%s)", argv[1:])
        sys.exit(EXIT_OK)


if __name__ == "__main__":
    Main.main(sys.argv)
